@@ -80,6 +80,9 @@ enum info_screen_geometry {
 /* Number of seconds to display a warning message outside ncurses mode */
 #define WARN_DISPLAY_DELAY	3
 
+/* Minimum SSID length */
+#define MAX_ESSID_LEN		16
+
 /*
  * Symbolic names of actions to take when crossing thresholds.
  * These actions invoke the corresponding ncurses functions.
@@ -104,12 +107,11 @@ static inline void threshold_action(enum threshold_action action)
  */
 enum scan_sort_order {
 	SO_CHAN,
-	SO_CHAN_REV,
 	SO_SIGNAL,
+	SO_ESSID,
 	SO_OPEN,
 	SO_CHAN_SIG,
-	SO_OPEN_SIG,
-	SO_OPEN_CH_SI
+	SO_OPEN_SIG
 };
 
 /*
@@ -134,7 +136,8 @@ extern struct wavemon_conf {
 	int	check_geometry,		/* ensure window is large enough */
 		cisco_mac,		/* Cisco-style MAC addresses */
 		random,			/* random signals */
-		override_bounds;	/* override autodetection */
+		override_bounds,	/* override autodetection */
+		scan_sort_asc;		/* direction of @scan_sort_order */
 
 	/* Enumerated values */
 	int	scan_sort_order,	/* channel|signal|open|chan/sig ... */
@@ -276,6 +279,7 @@ static inline int cp_from_scale(float value, const char *cscale, bool reverse)
 /*
  *	Wireless interfaces
  */
+extern const char *we_version(void);
 extern const char *conf_ifname(void);
 extern void conf_get_interface_list(bool init);
 extern char **iw_get_interface_list(void);
@@ -290,7 +294,7 @@ struct timer {
 };
 
 extern void start_timer(struct timer *t, unsigned long d);
-extern int end_timer(struct timer *t);
+extern bool end_timer(struct timer *t);
 
 /*
  *	Error handling
